      SUBROUTINE m_soil(mlon,kmb,itap,ptimestep,indice,mlev,
     $                  mtsrf,mtsoil, mfluxg1, 
     $                  A_sol,B_sol,zc,zd)
      USE dimphy
      IMPLICIT NONE
     

c=================================================================
c
c   Auteur:  Frederic Hourdin     30/01/92
c   Modifi\'e par Jean-Yves Grandpeix pour sol de montagne   17/11/2007
c   -------
c
c   objet:  computation of : the soil temperature evolution
c   ------  -                 the surfacic heat capacity "Capcal"
c                            the surface conduction flux mcapcal
c           for a mountain.
c
c
c   Method: implicit time integration
c   -------
c   Consecutive ground temperatures are related by:
c           T(k+1) = C(k) + D(k)*T(k)  (1)
c   the coefficients C and D are computed at the t-dt time-step.
c   Routine structure:
c   1)new temperatures are computed  using (1)
c   2)C and D coefficients are computed from the new temperature
c     profile for the t+dt time-step
c   3)the coefficients A and B are computed where the diffusive
c     fluxes at the t+dt time-step is given by
c            Fdiff = A + B Ts(t+dt)
c     or     Fdiff = F0 + Capcal (Ts(t+dt)-Ts(t))/dt
c            with F0 = A + B (Ts(t))
c                 Capcal = B*dt
c
c   Interface:
c   ----------
c
c   Arguments:
c   ----------
c   ptimestep                 physical timestep (s)
c   indice                    sub-surface index
c   snow(klon)                snow

c   mtsrf(klon,klev)          surface temperature at time-step t (K)
c   mtsoil(klon,klev,kmlay)   temperature inside the ground (K)
c   mcapcal(klon,klev)        surfacic specific heat (W*m-2*s*K-1)
c   mfluxg1(klon,klev)       surface diffusive flux from ground (Wm-2)
c
c=======================================================================
c   declarations:
c   -------------

#include "dimensions.h"
*KEEP,dimphy.
CC#include "dimphy.h"
*KEEP,mdimsoil.
#include "mdimsoil.h"
*KEEP,indicesol.
#include "indicesol.h"
#include "iniprint.h"
*KEND.

c-----------------------------------------------------------------------
c  arguments
c  ---------
   
      REAL ptimestep
      INTEGER indice
      INTEGER mlev(klon)
!      PARAMETER (mlev=4)

      REAL mtsrf(klon,klev)
      REAL mtsoil(klon,klev,kmlay)
      REAL snow(klon)
      REAL mcapcal(klon,klev),mfluxg1(klon,klev)
      REAL dfluxg1dTs,A_sol(klon,klev),B_sol(klon,klev)
   

c-----------------------------------------------------------------------
c  local arrays
c  ------------

      INTEGER ig,jk,im,itap,mlon,kmb(klon),iq
      REAL zdz2(kmlay),z1(klon,klev)
      REAL min_period,dalph_soil
      REAL ztherm_i(klon)

c   local saved variables:
c   ----------------------
      REAL dz1(kmlay),dz2(kmlay)
      REAL zc(klon,klev,kmlay),zd(klon,klev,kmlay)
      REAL lambda
      SAVE dz1,dz2,lambda
      LOGICAL firstcall
      SAVE firstcall
      REAL isol,isno,iice
      SAVE isol,isno,iice

      DATA firstcall/.true./

      DATA isol,isno,iice/2000.,2000.,2000./



c-----------------------------------------------------------------------
c   Depthts:
c   --------

      REAL fz,rk,fz1,rk1,rk2
      fz(rk)=fz1*(dalph_soil**rk-1.)/(dalph_soil-1.)
!
c   calcul de l'inertie thermique a partir de la variable rnat.
c   on initialise a iice meme au-dessus d'un point de mer au cas
c   ou le point de mer devienne point de glace au pas suivant
c   on corrige si on a un point de terre avec ou sans glace
c

!jyg#####################################################################        
      IF (itap .GT. -1000) THEN    ! suppression calcul
!jyg#####################################################################        
      IF (indice.EQ.is_sic) THEN
!
         DO ig = 1, klon
            ztherm_i(ig)   = iice
            IF (snow(ig).GT.0.0) ztherm_i(ig)   = isno
         ENDDO
!
      ELSE IF (indice.EQ.is_lic) THEN
!
         DO ig = 1, klon
            ztherm_i(ig)   = iice
            IF (snow(ig).GT.0.0) ztherm_i(ig)   = isno
         ENDDO
!
      ELSE IF (indice.EQ.is_ter) THEN
!
         DO iq = 1, mlon
             ig=kmb(iq)
            ztherm_i(ig)   = isol
            IF (snow(ig).GT.0.0) ztherm_i(ig)   = isno
         ENDDO
!
      ELSE IF (indice.EQ.is_oce) THEN
!
         DO ig = 1,klon
           
            ztherm_i(ig)   = iice
         ENDDO
!
      ELSE
!
         PRINT*, "valeur d indice non prevue", indice
         CALL abort
!
      ENDIF


!------------------------------------------ 
      IF (prt_level .ge. 20) THEN
       PRINT*,'indice surface',indice
      ENDIF
!
      IF (firstcall) THEN
!--------------------------
! initialisation
!---------------------------
           
            DO jk = 1,kmlay
             DO im = 1,klev
               DO iq = 1,mlon
                   ig=kmb(iq)
!
             !
                  mfluxg1(ig,im) = 0.
                  mcapcal(ig,im) = 0.
       
                  A_sol(ig,im) = 0.
                  B_sol(ig,im) = 0.

!              if(im.le.mlev(ig)) then
!                  mtsrf(ig,im) = 301.
!                  mtsoil(ig,im,jk) = 301.
!                else
!                 mtsrf(ig,im) = 0. 
!                 mtsoil(ig,im,jk) =0.
!              endif
!
             ENDDO ! jk = 1,kmlay
            ENDDO ! im = 1,klev
          ENDDO ! ig = 1,mlon
!
!
c-----------------------------------------------------------
c   ground levels
c   grnd=z/l where l is the skin depth of the diurnal cycle:
c   --------------------------------------------------------

         min_period=1800. ! en secondes
         dalph_soil=2.    ! rapport entre les epaisseurs de 2 couches succ.

         OPEN(99,file='soil.def',status='old',form='formatted',err=9999)
!
         READ(99,*) min_period
         READ(99,*) dalph_soil
!
      IF (prt_level .ge. 20) THEN
         PRINT*,'Discretization for the soil model'
         PRINT*,'First level e-folding depth',min_period,
     s   '   dalph',dalph_soil
      ENDIF
!
         CLOSE(99)
!
9999     CONTINUE

c   la premiere couche represente un dixieme de cycle diurne
         fz1=sqrt(min_period/3.14)
         
         DO jk=1,kmlay
!
            rk1=jk
            rk2=jk-1
            dz2(jk)=fz(rk1)-fz(rk2)
!
         ENDDO

!
         DO jk=1,kmlay-1
!
            rk1=jk+.5
            rk2=jk-.5
            dz1(jk)=1./(fz(rk1)-fz(rk2))
!
         ENDDO
!
         lambda=fz(.5)*dz1(1)
!
      IF (prt_level .ge. 20)  THEN
         PRINT*,'full layers, intermediate layers (seconds)'
      ENDIF
!
         DO jk=1,kmlay
            rk=jk
            rk1=jk+.5
            rk2=jk-.5
!
      IF (prt_level .ge. 20) THEN
            PRINT *,'fz=',
     .               fz(rk1)*fz(rk2)*3.14,fz(rk)*fz(rk)*3.14
      ENDIF
!
         ENDDO

         firstcall =.false.
c
c   FIN Initialisations
c   ----------------

      ELSE   ! 
c-----------------------------------------------------------------------
c   Computation of the soil temperatures using the Cgrd and Dgrd
c  coefficient computed at the previous time-step:
c  -----------------------------------------------

c    surface temperature
!
       DO im = 1,klev
         DO iq=1,mlon
             ig=kmb(iq)
!
            mtsoil(ig,im,1)=(lambda*zc(ig,im,1)+mtsrf(ig,im))/
     s      (lambda*(1.-zd(ig,im,1))+1.)
!

         ENDDO
       ENDDO

c   other temperatures
!
         DO jk=1,kmlay-1
          DO im = 1,klev
            DO iq=1,mlon
                  ig=kmb(iq)
!
               mtsoil(ig,im,jk+1)=zc(ig,im,jk)
     s                           +zd(ig,im,jk)*mtsoil(ig,im,jk)
!
            ENDDO
          ENDDO
         ENDDO
  
      ENDIF ! (firstcall)
c-----------------------------------------------------------------------
c   Computation of the Cgrd and Dgrd coefficient for the next step:
c   ---------------------------------------------------------------

      DO jk=1,kmlay
!
         zdz2(jk)=dz2(jk)/ptimestep
!
      ENDDO

      DO im = 1,klev
       DO iq=1,mlon
             ig=kmb(iq)

!
         z1(ig,im)=zdz2(kmlay)+dz1(kmlay-1)
!
         zc(ig,im,kmlay-1)=zdz2(kmlay)*mtsoil(ig,im,kmlay)/z1(ig,im)
!
         zd(ig,im,kmlay-1)=dz1(kmlay-1)/z1(ig,im)
!
       ENDDO
      ENDDO

      DO jk=kmlay-1,2,-1
       DO im = 1,klev
         DO iq=1,mlon
              ig=kmb(iq)
!
            z1(ig,im)=1./(zdz2(jk)+dz1(jk-1)
     s                   +dz1(jk)*(1.-zd(ig,im,jk)))
!
            zc(ig,im,jk-1)=
     s           (mtsoil(ig,im,jk)*zdz2(jk)
     s           +dz1(jk)*zc(ig,im,jk))*z1(ig,im)
                 zd(ig,im,jk-1)=dz1(jk-1)*z1(ig,im)
!
         ENDDO
       ENDDO
      ENDDO

c-----------------------------------------------------------------------
c   computation of the surface diffusive flux from ground and
c   calorific capacity of the ground:
c   ---------------------------------

      DO im = 1,klev
       DO iq=1,mlon
            ig=kmb(iq)
          if (im.le.mlev(ig)) then

         mfluxg1(ig,im)=ztherm_i(ig)*dz1(1)*
     s            (zc(ig,im,1)+(zd(ig,im,1)-1.)*mtsoil(ig,im,1))

         mcapcal(ig,im)=ztherm_i(ig)*
     s          (dz2(1)+ptimestep*(1.-zd(ig,im,1))*dz1(1))

         z1(ig,im)=lambda*(1.-zd(ig,im,1))+1.

         mcapcal(ig,im)=mcapcal(ig,im)/z1(ig,im)
!
        

         mfluxg1(ig,im)=mfluxg1(ig,im)
     s                  +mcapcal(ig,im)*(mtsoil(ig,im,1)*z1(ig,im)
     s                  -lambda*zc(ig,im,1)-mtsrf(ig,im))/ptimestep

!
!         B_sol(ig,im) = -mcapcal(ig,im)/ptimestep
!
!         A_sol(ig,im) = mfluxg1(ig,im)-B_sol(ig,im)*mtsrf(ig,im)
!
           endif
         ENDDO
       ENDDO
!Changement 13/08/2009, pour avoir mfluxg1 positive a midi,
!contrairement fluxg1 de grand d echelle
! i.e. on compte le flux mfluxg1 positivement vers le bas.
      DO im = 1,klev
       DO iq=1,mlon
            ig=kmb(iq)
          if (im.le.mlev(ig)) then
            mfluxg1(ig,im)= -mfluxg1(ig,im)
            B_sol(ig,im) = mcapcal(ig,im)/ptimestep
            A_sol(ig,im) = mfluxg1(ig,im)-B_sol(ig,im)*mtsrf(ig,im)
!mflux=A+B*mtsrf
          endif
         ENDDO
       ENDDO
!--------------------------------------------------------
!jyg#####################################################################        
      ENDIF    ! fin suppression
!jyg#####################################################################        

      RETURN
      END
